/*
 *		RStars.cpp
 *
 *		ScreenSaver addon.
 */

#include	"RStars.h"
#include	<InterfaceKit.h>
#include	<stdlib.h>

BScreenSaver *instantiate_screen_saver(BMessage *pMsg, image_id id)
{
	return new RStars(pMsg, id);
}


/*
 *		Constructor
 */
RStars::RStars(BMessage *pMsg, image_id id) : BScreenSaver(pMsg, id)
{
	// Nothing else to do here...
}


/*
 *		Destructor
 */
RStars::~RStars(void)
{
	// Nothing else to do here...
}


/*
 *		StartConfig
 */
void RStars::StartConfig(BView *pView)
{
	pView->AddChild(new BStringView(BRect(10, 10, 200, 35), B_EMPTY_STRING, "RStars v1.0.0 by Magnus Persson"));
	pView->AddChild(new BStringView(BRect(10, 40, 200, 65), B_EMPTY_STRING, "e-mail: magnus.persson@telia.com"));
}


/*
 *		StartSaver
 */
status_t RStars::StartSaver(BView *pView, bool /*bPreview*/)
{
	m_bInitScreenSaver = true;		// Clear screen and setup a few other things.
	SetTickSize(50000);
	return (B_OK);
}


/*
 *		Draw
 */
void RStars::Draw(BView *pView, int32 /*frame*/)
{
	if (m_bInitScreenSaver)
	{
		m_rc = pView->Bounds();
		m_width = m_rc.right - m_rc.left;
		m_height = m_rc.bottom - m_rc.top;

		// Randomize the star co-ordinates.
		for (uint16 i = 0; i < NUM_STARS; i++)
		{
			m_X[i] = (float)((int16)rand() % (uint16)(DISTANCE / 2));
			m_Y[i] = (float)((int16)rand() % (uint16)(DISTANCE / 2));
			m_Z[i] = (float)((int16)rand() % (uint16)DISTANCE);
		}

		// Fill the screen with black on the first frame:
		pView->SetLowColor(0, 0, 0);
		pView->FillRect(m_rc, B_SOLID_LOW);
		m_bInitScreenSaver = false;		// Only init once...
	}
	else
	{
		Clear2DLines(pView);

		for (uint16 i = 0; i < NUM_STARS; i++)
		{
			m_Z[i] -= STAR_SPEED;
			if (m_Z[i] < -DISTANCE)
				m_Z[i] = DISTANCE;
		}

		Project3Dto2D();
		Draw2DLines(pView);
	}
}


/*
 *		Project3Dto2D
 */
void RStars::Project3Dto2D(void)
{
	float	x, y, z;
	uint8	c;

	for (uint16 i = 0; i < NUM_STARS; i++)
	{
		z = DISTANCE + m_Z[i];
		if (z == 0)		// Prevent division by 0 error.
			z = 0.1;

		x = (m_X[i] * DISTANCE / z);
		y = (m_Y[i] * DISTANCE / z);

		m_2D[i].x = x + (m_width / 2);
		m_2D[i].y = y + (m_height / 2);

		// Calculate the color.
		c = 255 - (uint8)((m_Z[i] + DISTANCE) * 255.0 / (2 * DISTANCE));

		m_Color[i].red = c;
		m_Color[i].green = c;
		m_Color[i].blue = c;
		m_Color[i].alpha = 0;
	}
}


/*
 *		Clear2DLines
 */
void RStars::Clear2DLines(BView *pView)
{
	pView->BeginLineArray(NUM_STARS);
	for (uint16 i = 0; i < NUM_STARS; i++)
		pView->AddLine(m_2D[i], m_2D[i], COLOR_BLACK);
	pView->EndLineArray();
}


/*
 *		Draw2DLines
 */
void RStars::Draw2DLines(BView *pView)
{
	pView->BeginLineArray(NUM_STARS);
	for (uint16 i = 0; i < NUM_STARS; i++)
		pView->AddLine(m_2D[i], m_2D[i], m_Color[i]);
	pView->EndLineArray();
}
